=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-06-15 17:26:25 +0000
+++ bzrlib/branch.py	2006-06-15 20:57:44 +0000
@@ -1188,10 +1188,14 @@
         assert self.base[-1] == '/'
         for l in _locs:
             try:
-                return urlutils.join(self.base[:-1], 
-                            self.control_files.get(l).read().strip('\n'))
+                parent = self.control_files.get(l).read().strip('\n')
             except NoSuchFile:
-                pass
+                continue
+            # This is an old-format absolute path to a local branch
+            # turn it into a url
+            if parent.startswith('/'):
+                parent = urlutils.local_path_to_url(parent.decode('utf8'))
+            return urlutils.join(self.base[:-1], parent)
         return None
 
     def get_push_location(self):

=== modified file 'bzrlib/tests/branch_implementations/test_parent.py'
--- bzrlib/tests/branch_implementations/test_parent.py	2006-06-06 22:37:30 +0000
+++ bzrlib/tests/branch_implementations/test_parent.py	2006-06-15 19:13:50 +0000
@@ -15,7 +15,9 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+import cStringIO
 import os
+import sys
 
 from bzrlib.branch import Branch
 import bzrlib.errors
@@ -32,34 +34,43 @@
     def test_no_default_parent(self):
         """Branches should have no parent by default"""
         b = self.make_branch('.')
-        self.assertEquals(b.get_parent(), None)
+        self.assertEqual(None, b.get_parent())
         
     def test_set_get_parent(self):
         """Set, re-get and reset the parent"""
         b = self.make_branch('.')
         url = 'http://bazaar-vcs.org/bzr/bzr.dev'
         b.set_parent(url)
-        self.assertEquals(b.get_parent(), url)
-        self.assertEqual(b.control_files.get('parent').read().strip('\n'), url)
+        self.assertEqual(url, b.get_parent())
+        self.assertEqual(url, b.control_files.get('parent').read().strip('\n'))
 
         b.set_parent(None)
-        self.assertEquals(b.get_parent(), None)
+        self.assertEqual(None, b.get_parent())
 
         b.set_parent('../other_branch')
         cwd = getcwd()
 
-        self.assertEquals(b.get_parent(), local_path_to_url('../other_branch'))
+        self.assertEqual(local_path_to_url('../other_branch'), b.get_parent())
         path = local_path_to_url('../yanb')
         b.set_parent(path)
-        self.assertEqual(b.control_files.get('parent').read().strip('\n'), 
-            '../yanb')
-        self.assertEqual(b.get_parent(), path)
+        self.assertEqual('../yanb',
+            b.control_files.get('parent').read().strip('\n'))
+        self.assertEqual(path, b.get_parent())
 
 
         self.assertRaises(bzrlib.errors.InvalidURL, b.set_parent, u'\xb5')
         b.set_parent(escape(u'\xb5'))
-        self.assertEqual(b.control_files.get('parent').read().strip('\n'), 
-            '%C2%B5')
-
-        self.assertEqual(b.get_parent(), b.base + '%C2%B5')
+        self.assertEqual('%C2%B5', 
+            b.control_files.get('parent').read().strip('\n'))
+
+        self.assertEqual(b.base + '%C2%B5', b.get_parent())
+
+        # Handle the case for older style absolute local paths
+        if sys.platform == 'win32':
+            # TODO: jam 20060515 Do we want to special case Windows local
+            #       paths as well? Nobody has complained about it.
+            pass
+        else:
+            b.control_files.put('parent', cStringIO.StringIO('/local/abs/path'))
+            self.assertEqual('file:///local/abs/path', b.get_parent())
 

=== modified file 'bzrlib/tests/test_urlutils.py'
--- bzrlib/tests/test_urlutils.py	2006-06-15 01:58:02 +0000
+++ bzrlib/tests/test_urlutils.py	2006-06-15 21:54:28 +0000
@@ -209,6 +209,12 @@
         test('http://bar', 'http://foo', 'http://bar')
         test('sftp://bzr/foo', 'http://foo', 'bar', 'sftp://bzr/foo')
         test('file:///bar', 'foo', 'file:///bar')
+
+        # From a base path
+        test('file:///foo', 'file:///', 'foo')
+        test('file:///bar/foo', 'file:///bar/', 'foo')
+        test('http://host/foo', 'http://host/', 'foo')
+        test('http://host/', 'http://host', '')
         
         # Invalid joinings
         # Cannot go above root

=== modified file 'bzrlib/urlutils.py'
--- bzrlib/urlutils.py	2006-06-15 01:58:02 +0000
+++ bzrlib/urlutils.py	2006-06-15 21:07:12 +0000
@@ -116,6 +116,11 @@
     if m:
         scheme = m.group('scheme')
         path = m.group('path').split('/')
+        if path[-1:] == ['']:
+            # Strip off a trailing slash
+            # This helps both when we are at the root, and when
+            # 'base' has an extra slash at the end
+            path = path[:-1]
     else:
         path = base.split('/')
 

